@@ -17,8 +17,8 @@ use Symfony\Component\Config\Loader\LoaderInterface;
 use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
 use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\ContainerInterface;
-use Symfony\Component\DependencyInjection\Reference;
 use Symfony\Component\DependencyInjection\Extension\ExtensionInterface;
+use Symfony\Component\DependencyInjection\Reference;
 use Symfony\Component\Filesystem\Exception\IOException;
 use Symfony\Component\Filesystem\Filesystem;
 use Symfony\Component\HttpFoundation\Request;
@@ -142,7 +142,7 @@ class KernelTest extends TestCase
     public function testBootSetsTheBootedFlagToTrue()
     {
         // use test kernel to access isBooted()
-        $kernel = $this->getKernel(['initializeBundles']);
+        $kernel = new KernelForTest('test', false);
         $kernel->boot();
 
         $this->assertTrue($kernel->isBooted());
@@ -173,7 +173,7 @@ class KernelTest extends TestCase
         $bundle->expects($this->once())
             ->method('shutdown');
 
-        $kernel = $this->getKernel([], [$bundle]);
+        $kernel = new KernelForTest('test', false, true, [$bundle]);
 
         $kernel->boot();
         $kernel->shutdown();
@@ -191,10 +191,7 @@ class KernelTest extends TestCase
             })
         ;
 
-        $kernel = $this->getKernel(['getBundles']);
-        $kernel->expects($this->any())
-            ->method('getBundles')
-            ->willReturn([$bundle]);
+        $kernel = new KernelForTest('test', false, true, [$bundle]);
 
         $kernel->boot();
         $kernel->shutdown();
@@ -228,9 +225,7 @@ class KernelTest extends TestCase
         $catch = true;
         $request = new Request();
 
-        $httpKernelMock = $this->getMockBuilder(HttpKernel::class)
-            ->disableOriginalConstructor()
-            ->getMock();
+        $httpKernelMock = $this->createStub(HttpKernelInterface::class);
 
         $kernel = $this->getKernel(['getHttpKernel', 'boot']);
         $kernel->expects($this->once())
@@ -255,29 +250,31 @@ class KernelTest extends TestCase
     public function testLocateResourceThrowsExceptionWhenNameIsNotValid()
     {
         $this->expectException(\InvalidArgumentException::class);
-        $this->getKernel()->locateResource('Foo');
+        (new KernelForTest('test', false))->locateResource('Foo');
     }
 
     public function testLocateResourceThrowsExceptionWhenNameIsUnsafe()
     {
         $this->expectException(\RuntimeException::class);
-        $this->getKernel()->locateResource('@FooBundle/../bar');
+        (new KernelForTest('test', false))->locateResource('@FooBundle/../bar');
     }
 
     public function testLocateResourceThrowsExceptionWhenBundleDoesNotExist()
     {
         $this->expectException(\InvalidArgumentException::class);
-        $this->getKernel()->locateResource('@FooBundle/config/routing.xml');
+        (new KernelForTest('test', false))->locateResource('@FooBundle/config/routing.xml');
     }
 
     public function testLocateResourceThrowsExceptionWhenResourceDoesNotExist()
     {
+        $bundle = $this->createStub(BundleInterface::class);
+        $bundle->method('getPath')->willReturn(__DIR__.'/Fixtures/Bundle1Bundle');
         $this->expectException(\InvalidArgumentException::class);
         $kernel = $this->getKernel(['getBundle']);
         $kernel
             ->expects($this->once())
             ->method('getBundle')
-            ->willReturn($this->getBundle(__DIR__.'/Fixtures/Bundle1Bundle'))
+            ->willReturn($bundle)
         ;
 
         $kernel->locateResource('@Bundle1Bundle/config/routing.xml');
@@ -285,11 +282,13 @@ class KernelTest extends TestCase
 
     public function testLocateResourceReturnsTheFirstThatMatches()
     {
+        $bundle = $this->createStub(BundleInterface::class);
+        $bundle->method('getPath')->willReturn(__DIR__.'/Fixtures/Bundle1Bundle');
         $kernel = $this->getKernel(['getBundle']);
         $kernel
             ->expects($this->once())
             ->method('getBundle')
-            ->willReturn($this->getBundle(__DIR__.'/Fixtures/Bundle1Bundle'))
+            ->willReturn($bundle)
         ;
 
         $this->assertEquals(__DIR__.'/Fixtures/Bundle1Bundle/foo.txt', $kernel->locateResource('@Bundle1Bundle/foo.txt'));
@@ -297,11 +296,14 @@ class KernelTest extends TestCase
 
     public function testLocateResourceOnDirectories()
     {
+        $bundle = $this->createStub(BundleInterface::class);
+        $bundle->method('getName')->willReturn('Bundle1Bundle');
+        $bundle->method('getPath')->willReturn(__DIR__.'/Fixtures/Bundle1Bundle');
         $kernel = $this->getKernel(['getBundle']);
         $kernel
             ->expects($this->exactly(2))
             ->method('getBundle')
-            ->willReturn($this->getBundle(__DIR__.'/Fixtures/Bundle1Bundle', null, 'Bundle1Bundle'))
+            ->willReturn($bundle)
         ;
 
         $this->assertEquals(
@@ -318,10 +320,12 @@ class KernelTest extends TestCase
     {
         $this->expectException(\LogicException::class);
         $this->expectExceptionMessage('Trying to register two bundles with the same name "DuplicateName"');
-        $fooBundle = $this->getBundle(__DIR__.'/Fixtures/FooBundle', 'FooBundle', 'DuplicateName');
-        $barBundle = $this->getBundle(__DIR__.'/Fixtures/BarBundle', 'BarBundle', 'DuplicateName');
+        $fooBundle = $this->createStub(BundleInterface::class);
+        $fooBundle->method('getName')->willReturn('DuplicateName');
+        $barBundle = $this->createStub(BundleInterface::class);
+        $barBundle->method('getName')->willReturn('DuplicateName');
 
-        $kernel = $this->getKernel([], [$fooBundle, $barBundle]);
+        $kernel = new KernelForTest('test', false, true, [$fooBundle, $barBundle]);
         $kernel->boot();
     }
 
@@ -420,7 +424,7 @@ class KernelTest extends TestCase
             }
 
             /**
-             * @deprecated since Symfony 7.4, to be removed in Symfony 8.0 together with XML support.
+             * To be removed when symfony/dependency-injection is bumped to 8.0+.
              */
             public function getNamespace(): string
             {
@@ -428,7 +432,7 @@ class KernelTest extends TestCase
             }
 
             /**
-             * @deprecated since Symfony 7.4, to be removed in Symfony 8.0 together with XML support.
+             * To be removed when symfony/dependency-injection is bumped to 8.0+.
              */
             public function getXsdValidationBasePath(): string|false
             {
@@ -527,10 +531,33 @@ class KernelTest extends TestCase
         $this->assertSame(1, ResettableService::$counter);
     }
 
+    public function testHttpCacheHandlesRequestsAfterKernelBoot()
+    {
+        $kernel = new CustomProjectDirKernel(static function (ContainerBuilder $container) {
+            $container->register('http_cache', RecordingHttpCache::class)
+                ->setPublic(true);
+        }, new ThrowingHttpKernel(), 'http_cache_worker');
+
+        $kernel->boot();
+
+        $firstResponse = $kernel->handle(Request::create('/worker-first'));
+        $secondResponse = $kernel->handle(Request::create('/worker-second'));
+
+        /** @var RecordingHttpCache $httpCache */
+        $httpCache = $kernel->getContainer()->get('http_cache');
+
+        $this->assertSame([
+            ['/worker-first', HttpKernelInterface::MAIN_REQUEST],
+            ['/worker-second', HttpKernelInterface::MAIN_REQUEST],
+        ], $httpCache->handledPaths);
+        $this->assertSame('cached: /worker-first', $firstResponse->getContent());
+        $this->assertSame('cached: /worker-second', $secondResponse->getContent());
+    }
+
     #[Group('time-sensitive')]
     public function testKernelStartTimeIsResetWhileBootingAlreadyBootedKernel()
     {
-        $kernel = $this->getKernel(['initializeBundles'], [], true);
+        $kernel = new KernelForTest('test', true);
         $kernel->boot();
         $preReBoot = $kernel->getStartTime();
 
@@ -580,37 +607,6 @@ class KernelTest extends TestCase
         }
     }
 
-    /**
-     * Returns a mock for the BundleInterface.
-     */
-    protected function getBundle($dir = null, $className = null, $bundleName = null): BundleInterface
-    {
-        $bundle = $this
-            ->getMockBuilder(BundleInterface::class)
-            ->disableOriginalConstructor()
-        ;
-
-        if ($className) {
-            $bundle->setMockClassName($className);
-        }
-
-        $bundle = $bundle->getMock();
-
-        $bundle
-            ->expects($this->any())
-            ->method('getName')
-            ->willReturn($bundleName ?? $bundle::class)
-        ;
-
-        $bundle
-            ->expects($this->any())
-            ->method('getPath')
-            ->willReturn($dir)
-        ;
-
-        return $bundle;
-    }
-
     /**
      * Returns a mock for the abstract kernel.
      *
@@ -628,7 +624,7 @@ class KernelTest extends TestCase
         ;
 
         $kernel = $kernelMockBuilder->getMock();
-        $kernel->expects($this->any())
+        $kernel
             ->method('registerBundles')
             ->willReturn($bundles)
         ;
@@ -719,54 +715,6 @@ class PassKernel extends CustomProjectDirKernel implements CompilerPassInterface
     }
 }
 
-class KernelForTest extends Kernel
-{
-    public function __construct(string $environment, bool $debug, private readonly bool $fakeContainer = true)
-    {
-        parent::__construct($environment, $debug);
-    }
-
-    public function getBundleMap(): array
-    {
-        return [];
-    }
-
-    public function registerBundles(): iterable
-    {
-        return [];
-    }
-
-    public function registerContainerConfiguration(LoaderInterface $loader): void
-    {
-    }
-
-    public function isBooted(): bool
-    {
-        return $this->booted;
-    }
-
-    public function getProjectDir(): string
-    {
-        return __DIR__;
-    }
-
-    protected function initializeContainer(): void
-    {
-        if ($this->fakeContainer) {
-            $this->container = new ContainerBuilder();
-        } else {
-            parent::initializeContainer();
-        }
-    }
-}
-
-class KernelForTestWithLoadClassCache extends KernelForTest
-{
-    public function doLoadClassCache(): void
-    {
-    }
-}
-
 class FragmentHandlingKernel implements HttpKernelInterface
 {
     public array $handledPaths = [];
@@ -802,3 +750,75 @@ class FragmentRenderingHttpCache implements HttpKernelInterface
         return new Response(implode('', array_map(static fn (Response $response) => $response->getContent(), $responses)));
     }
 }
+
+class RecordingHttpCache implements HttpKernelInterface
+{
+    public array $handledPaths = [];
+
+    public function handle(Request $request, int $type = self::MAIN_REQUEST, bool $catch = true): Response
+    {
+        $this->handledPaths[] = [$request->getPathInfo(), $type];
+
+        return new Response('cached: '.$request->getPathInfo());
+    }
+}
+
+class ThrowingHttpKernel implements HttpKernelInterface
+{
+    public function handle(Request $request, int $type = self::MAIN_REQUEST, bool $catch = true): Response
+    {
+        throw new \LogicException('The worker HTTP kernel should not be reached when the http_cache service handles the request.');
+    }
+}
+
+class KernelForTest extends Kernel
+{
+    public function __construct(
+        string $environment,
+        bool $debug,
+        private readonly bool $fakeContainer = true,
+        private array $registeredBundles = [],
+    ) {
+        parent::__construct($environment, $debug);
+    }
+
+    public function getBundleMap(): array
+    {
+        return [];
+    }
+
+    public function registerBundles(): iterable
+    {
+        return $this->registeredBundles;
+    }
+
+    public function registerContainerConfiguration(LoaderInterface $loader): void
+    {
+    }
+
+    public function isBooted(): bool
+    {
+        return $this->booted;
+    }
+
+    public function getProjectDir(): string
+    {
+        return __DIR__;
+    }
+
+    protected function initializeContainer(): void
+    {
+        if ($this->fakeContainer) {
+            $this->container = new ContainerBuilder();
+        } else {
+            parent::initializeContainer();
+        }
+    }
+}
+
+class KernelForTestWithLoadClassCache extends KernelForTest
+{
+    public function doLoadClassCache(): void
+    {
+    }
+}
